(*********************************************************************)
(*                                                                   *)
(*   THIS MODULE IS A COMPONENT OF DOOR SKELETON V1.0                *)
(*   COPYRIGHT (C) 1992 BERNIE GALLAGHER                             *)
(*                                                                   *)
(*********************************************************************)
(*                                                                   *)
(*   COMMUNICATE WITH THE MODEM VIA THE SERIAL PORT                  *)
(*                                                                   *)
(*********************************************************************)
CONST SAVE_REGS:              INTEGER = 0;
CONST BUFFER_SIZE             = 4096;
VAR BUFFER_QUEUE:             ARRAY [0..BUFFER_SIZE] OF CHAR;
VAR PORT_ADDRESS:             INTEGER;
VAR INTERRUPT_LEVEL:          INTEGER;
VAR QUEUE_IN:                 INTEGER;
VAR QUEUE_OUT:                INTEGER;

FUNCTION ASYNC_PENDING : BOOLEAN;
BEGIN

    ASYNC_PENDING := (QUEUE_OUT <> QUEUE_IN);
    END;

PROCEDURE ASYNC_CLEAR;
BEGIN

    INLINE ($FA);
    QUEUE_IN := 0;
    QUEUE_OUT := QUEUE_IN;
    INLINE ($FB);
    END;

PROCEDURE ASYNC_ISR;
BEGIN

    INLINE ($50);                      { PUSH AX                 }
    INLINE ($53);                      { PUSH BX                 }
    INLINE ($52);                      { PUSH DX                 }
    INLINE ($1E);                      { PUSH DS                 }
    INLINE ($FB);                      { STI                     }
    INLINE ($2E);                      { CS:                     }
    INLINE ($FF/$36/SAVE_REGS);        { PUSH SAVE_REGS          }
    INLINE ($1F);                      { POP  DS                 }
    INLINE ($8B/$16/PORT_ADDRESS);     { MOV  DX,PORT_ADDRESS    }
    INLINE ($EC);                      { IN   AL,DX              }
    INLINE ($8B/$1E/QUEUE_IN);         { MOV  BX,QUEUE_IN        }
    INLINE ($88/$87/BUFFER_QUEUE);     { MOV  BX+BUFFER_QUEUE,AL }
    INLINE ($43);                      { INC  BX                 }
    INLINE ($81/$FB/BUFFER_SIZE);      { CMP  BX,BUFFER_SIZE     }
    INLINE ($7E/$02);                  { JLE  0121               }
    INLINE ($33/$DB);                  { XOR  BX,BX              }
    INLINE ($3B/$1E/QUEUE_OUT);        { CMP  BX,QUEUE_OUT       }
    INLINE ($74/$04);                  { JZ   012B               }
    INLINE ($89/$1E/QUEUE_IN);         { MOV  QUEUE_IN,BX        }
    INLINE ($FA);                      { CLI                     }
    INLINE ($B0/$20);                  { MOV  AL,20              }
    INLINE ($E6/$20);                  { OUT  20,AL              }
    INLINE ($1F);                      { POP  DS                 }
    INLINE ($5A);                      { POP  DX                 }
    INLINE ($5B);                      { POP  BX                 }
    INLINE ($58);                      { POP  AX                 }
    INLINE ($5C);                      { POP  SP                 }
    INLINE ($5D);                      { POP  BP                 }
    INLINE ($CF);                      { IRET                    }
    END;

PROCEDURE ASYNC_CLOSE;
VAR INPORT:                   INTEGER;
VAR MASK:                     INTEGER;
BEGIN

    INLINE ($FA);
    INPORT := PORT [$21];
    MASK := 1 SHL INTERRUPT_LEVEL;
    PORT [$21] := INPORT OR MASK;
    PORT [PORT_ADDRESS + 2] := 0;
    PORT [PORT_ADDRESS + 4] := PORT [PORT_ADDRESS + 4] AND 1;
    INLINE ($FB);
    END;

PROCEDURE ASYNC_READY (READY_PARM: BOOLEAN);
VAR READY_FLAG:                    BYTE;
BEGIN

    READY_FLAG := PORT [PORT_ADDRESS + 4] AND $FE;
    IF READY_PARM THEN READY_FLAG := READY_FLAG + 1;
    PORT [PORT_ADDRESS + 4] := READY_FLAG;
    END;

PROCEDURE ASYNC_SET_BAUD (BAUD: INTEGER);
VAR SPEED_MASK:                 BYTE;
BEGIN

    IF (BAUD >= 300) AND (BAUD <= 9600) THEN BEGIN

        BAUD := TRUNC (115200.0 / BAUD);
        SPEED_MASK := PORT [PORT_ADDRESS + 3] OR 128;
        PORT [PORT_ADDRESS + 3] := SPEED_MASK;
        PORT [PORT_ADDRESS] := LO (BAUD);
        PORT [PORT_ADDRESS + 1] := HI (BAUD);
        PORT [PORT_ADDRESS + 3] := SPEED_MASK AND 127;
        END;

    END;

FUNCTION ASYNC_CARRIER: BOOLEAN;
BEGIN

    ASYNC_CARRIER := (PORT [PORT_ADDRESS + 6] AND 128) <> 0;
    END;

FUNCTION ASYNC_OPEN (COMPORT, BAUD: INTEGER): BOOLEAN;
VAR PORT_HOLD:                                INTEGER;
VAR MASK:                                     INTEGER;
VAR RESULT:                                   BOOLEAN;
VAR REGS:                                     REGISTERS;
BEGIN

    RESULT := FALSE;
    SAVE_REGS := DSEG;
    IF COMPORT = 1 THEN BEGIN

        PORT_ADDRESS := $3F8;
        INTERRUPT_LEVEL := 4;
        RESULT := TRUE;
        END

    ELSE IF COMPORT = 2 THEN BEGIN

        PORT_ADDRESS := $2F8;
        INTERRUPT_LEVEL := 3;
        RESULT := TRUE;
        END

    ELSE IF COMPORT = 3 THEN BEGIN

        PORT_ADDRESS := $3E8;
        INTERRUPT_LEVEL := 4;
        RESULT := TRUE;
        END

    ELSE IF COMPORT = 4 THEN BEGIN

        PORT_ADDRESS := $2E8;
        INTERRUPT_LEVEL := 3;
        RESULT := TRUE;
        END;

    IF RESULT THEN IF (PORT [PORT_ADDRESS + 2] AND $00F8) <> 0 THEN BEGIN

        RESULT := FALSE;
        END

    ELSE BEGIN

        ASYNC_CLEAR;
        PORT [PORT_ADDRESS + 3] := $03;
        REGS.AX := $2500 + ((INTERRUPT_LEVEL + 8) AND $00FF);
        REGS.DS := CSEG;
        REGS.DX := OFS (ASYNC_ISR);
        MSDOS (REGS);
        INLINE ($FA);
        PORT_HOLD := PORT [PORT_ADDRESS + 5];
        PORT_HOLD := PORT [PORT_ADDRESS];
        PORT_HOLD := PORT [$21];
        MASK := (1 SHL INTERRUPT_LEVEL) XOR $00FF;
        PORT [$21] := PORT_HOLD AND MASK;
        PORT [PORT_ADDRESS + 1] := $01;
        PORT_HOLD := PORT [PORT_ADDRESS + 4];
        PORT [PORT_ADDRESS + 4] := PORT_HOLD OR $08;
        ASYNC_READY (TRUE);
        INLINE ($FB);
        ASYNC_CLEAR;
        ASYNC_SET_BAUD (BAUD);
        RESULT := TRUE;
        END;

    ASYNC_OPEN := RESULT;
    END;

FUNCTION ASYNC_RECEIVE (VAR IN_CHAR: CHAR): BOOLEAN;
BEGIN

    IF ASYNC_PENDING THEN BEGIN

        INLINE ($FA);
        IN_CHAR := BUFFER_QUEUE [QUEUE_OUT];
        QUEUE_OUT := (QUEUE_OUT + 1) MOD (BUFFER_SIZE + 1);
        INLINE ($FB);
        ASYNC_RECEIVE := TRUE;
        END

    ELSE BEGIN

        IN_CHAR := CHR (0);
        ASYNC_RECEIVE := FALSE;
        END;

    END;

PROCEDURE ASYNC_SEND (OUTPUT_STRING: STRING255);
VAR SEND_CHAR:                       INTEGER;
BEGIN

    FOR SEND_CHAR := 1 TO LENGTH (OUTPUT_STRING) DO BEGIN

        WHILE (PORT [PORT_ADDRESS + 5] AND 32) = 0 DO;
        PORT [PORT_ADDRESS] := ORD (OUTPUT_STRING [SEND_CHAR]);
        END;

    END;

